from os import path

import sqlite3
import pandas as pd

from bokeh.plotting import figure
from bokeh.models import ColumnDataSource
from bokeh.models.layouts import TabPanel
from bokeh.models.widgets import Select
from bokeh.layouts import column, row


def visitor_count():
    """ Create a tab with an interactive histogram of visitor numbers
    :return: bokeh TabPanel object 
    """
    appDirpath = path.basename(path.split(path.dirname(__file__))[0])
    dbFilepath = path.join(appDirpath, 'data', 'dinofun.db')
    #dbFilepath = path.join(appDirpath, 'data', 'minifun.db')
    
    binSizeDict = {
        '---': None,
        '10 min': 10, '15 min': 15, '20 min': 20, '30 min': 30,
        '1 h': 60, '2 h': 120
    }
    days = ('Fri', 'Sat', 'Sun')
    
    
    def make_plot(src, legendLabel='Fri'):
        """ Create the figure with the histogram
        :return: bokeh Figure object 
        """
        p = figure(width=800, height=300, x_axis_type='datetime')
        # VBar.x refers to the center of the bar
        p.vbar(source=src, x='binCen', top='count', width='width', legend_label='Fri')
        # TODO
        return p
    
    
    def load_source(binSizeStr='---', day='Fri'):
        """ Load trajectory data from the database
        :param binSizeStr: a key from `binSizeDict`
        :param day: day to be queried
        :return: bokeh ColumnDataSource object 
        """
        if binSizeStr == "---":
            return ColumnDataSource({ 'binCen': [], 'count': [], 'width': [] })
        
        table = 'movement' + day
        binSizeInMinutes = binSizeDict[binSizeStr]
        # Group the date by temporal bins for a histogram.
        # Notice that the month (June) is given as "6", not "06" in the table.
        # `substr(timestamp,-8)` selects the time (not the date) part of the *timestamp* column.
        # `strftime("%s",foo)` formats the string foo as seconds since 1970-01-01.
        # The binning happens through rounding the seconds to the specified minutes.
        # After the following query, column binCen has values that could be called binMin;
        # it is easier to fix this in a second step.
        q = ('SELECT min(timestamp) binCen, count(distinct id) count '
             'FROM ' + table + ' WHERE type="movement" '
             'GROUP BY cast((strftime("%s",substr(timestamp,-8))/60/?) AS INT);')
        df = None
        try:
            with sqlite3.connect(dbFilepath) as con:
                df = pd.read_sql_query(q, con, params=(binSizeInMinutes,), parse_dates=['binCen'])
        except sqlite3.Error as e:
            print('Visitor count: load_source: sqlite3.Error:', e)
            return ColumnDataSource()
                
        # binMin to binCen
        df['binCen'] = df.binCen + pd.Timedelta(str(binSizeInMinutes/2) + ' min')
        
        # constant width of bars in milliseconds
        width = binSizeInMinutes*60*1000
        df['width'] = [width]*len(df)
        
        print(day, '-'*76)
        print(df)
        return ColumnDataSource(df)
    
    
    def update(attr, old, new):
        """ Update the data after a user interaction
        """
        binSize = selectBinSize.value
        print('Visitor count: update: binSize =', binSize)
        srcNew = load_source(binSize)
        srcCur.data.update(srcNew.data)
        # TODO possibly a good place for extension
    
    
    # Create the user controls and link the interactions         
    selectBinSize = Select(title='Bin size:', value="---", options=list(binSizeDict.keys()))
    selectBinSize.on_change('value', update)
    
    controls = column(selectBinSize, width=200)
    
    # Create the plot
    srcCur = load_source()
    p = make_plot(srcCur, 'Fri')
    
    # Create a layout for the tab and initialize it
    layout = row(controls, p)
    # TODO possibly a good place for extension
    tab = TabPanel(child=layout, title='Visitor count')
    
    return tab
  
