from os import path

import sqlite3
import pandas as pd

from bokeh.plotting import figure
from bokeh.models import ColumnDataSource
from bokeh.models.layouts import TabPanel
from bokeh.layouts import column, row


def visit_timespan():
    """ Create a tab for visitors' park time analysis
    :return: bokeh TabPanel object
    """
    appDirpath = path.basename(path.split(path.dirname(__file__))[0])
    dbFilepath = path.join(appDirpath, 'data', 'dinofun.db')
    
    
    def make_scatterplot(src):
        """ Create a scatterplot for the arrival/departure date
        :param src: ColumnDataSource with arrival and departure times
        :return: bokeh Figure object
        """
        p = figure(x_axis_type='datetime', y_axis_type='datetime')
        p.circle(source=src, x='arrival', y='departure', color='red')
        p.xaxis.axis_label = "Arrival on Friday"
        p.yaxis.axis_label = "Departure on Friday"
        return p
    
    
    def make_histogram(src):
        ''' Create a histogram of visitors' time in the park
        :param src: ColumnDataSource with arrival and departure times
        :return: bokeh Figure object
        '''
        p = figure(width=300, height=600, y_axis_type='datetime')
        
        # create dummy data for demo purposes, use src instead
        timespan = [pd.Timedelta('1h'), pd.Timedelta('10h')]
        count = [3, 7]
        src = ColumnDataSource({'timespan': timespan, 'count': count})
        # TODO
        
        p.hbar(source=src, y="timespan", right="count", left=0, height=60*60*1000,
               fill_color="#b3de69")
        return p
    
    
    def load_source(day='Fri'):
        """ Load visitor park times from the database
        :param day: day to be queried
        :return: bokeh ColumnDataSource object
        """
        table = 'movement' + day
        q = ('SELECT timestamp AS arrival, timestamp AS departure FROM ' + table + ' LIMIT 100')
        # TODO
        df = None
        try:
            with sqlite3.connect(dbFilepath) as con:
                df = pd.read_sql_query(q, con, parse_dates=['arrival', 'departure'])
        except sqlite3.Error as e:
            print('Visit timespan: load_source: sqlite3.Error:', e)
            return ColumnDataSource()
        
        print(day, '-'*76)
        print(df)
        return ColumnDataSource(df)
    
    
    # Create the plot
    src = load_source()
    scatterPlot = make_scatterplot(src)
    histogram = make_histogram(src)
    
    # Create a layout for the tab and initialize it
    layout = row(scatterPlot, histogram)
    tab = TabPanel(child=layout, title='Visit timespan')
    return tab
  
